@@ -78,7 +78,8 @@
  * Example usage:
  *
  * //Global init requiring JSON serializer
- * if (data_init(MIME_TYPE_JSON_PLUGIN, NULL)) fatal("failed");
+ * if (data_init()) fatal("failed");
+ * if (serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL)) fatal("failed");
  * //Create root data entry:
  * data_t *ex = data_new();
  * //Set data entry to be a dictionary type
@@ -89,13 +90,13 @@
  * data_set_int(data_key_set(ex, "test2"), 12345);
  * // serialize into JSON string
  * char *json = NULL;
- * data_g_serialize(&json, ex, MIME_TYPE_JSON, DATA_SER_FLAGS_PRETTY);
+ * serialize_g_data_to_string(&json, &length, ex, MIME_TYPE_JSON, SER_FLAGS_PRETTY);
  * // cleanup the example
  * FREE_NULL_DATA(ex);
  * // log the json
  * debug("example json: %s", json);
  * // deserialise the JSON back into data_t
- * data_g_deserialize(&ex, json, strlen(json), MIME_TYPE_JSON);
+ * serialize_g_string_to_data(&ex, json, strlen(json), MIME_TYPE_JSON);
  * xfree(json);
  * // verify contents
  * xassert(data_get_type(ex) == DATA_TYPE_DICT);
@@ -202,21 +203,13 @@ typedef data_for_each_cmd_t (*DataDictForF) (const char *key, data_t *data, void
 typedef data_for_each_cmd_t (*DataDictForFConst) (const char *key, const data_t *data, void *arg);
 
 /*
- * Initialize static structs needed by data functions and
- * 	load serializer plugins
+ * Initialize static structs needed by data functions
  *
- * It is safe to call this function multiple times with different plugins as
- * they will be loaded only once.
- *
- * IN plugins - comma delimited list of plugins or "list"
- * 	pass NULL to load all found or "" to load none of them
- *
- * IN listf - function to call if plugins="list" (may be NULL)
  * RET SLURM_SUCCESS or error
  */
-extern int data_init(const char *plugins, plugrack_foreach_t listf);
+extern int data_init(void);
 /*
- * Cleanup global memory used by data_t helpers and unload all plugins
+ * Cleanup global memory used by data_t helpers
  *
  * WARNING: must be called only once after all data commands complete
  */
@@ -438,6 +431,21 @@ extern int data_list_for_each(data_t *d, DataListForF f, void *arg);
  */
 extern int data_list_for_each_const(const data_t *d, DataListForFConst f, void *arg);
 
+/*
+ * Use match() function ptr find a specific matching value in list
+ * IN data - data list to find entity with given needle
+ * IN match - strict weak ordering function that compares two data to arg
+ *   returning true if the first precedes the second.
+ * IN needle - match against needle value
+ * NOTE: the lists are currently unordered but this is being implemented
+ *   as strict weak order to allow log(n) searching in the future.
+ * RET ptr to matching data or NULL if match not found
+ */
+extern data_t *data_list_find_first(
+	data_t *data,
+	bool (*match)(const data_t *data, void *needle),
+	void *needle);
+
 /*
  *  For each item in data dictionary [d], invokes the function [f] with [arg].
  *
@@ -479,6 +487,14 @@ extern data_t *data_list_append(data_t *data);
  */
 extern data_t *data_list_prepend(data_t *data);
 
+/*
+ * Extract first data from start of list
+ * IN data data object (list type only)
+ * RET first data start of list or NULL if list is empty
+ * 	Result must be free'd up with FREE_NULL_DATA()
+ */
+extern data_t *data_list_dequeue(data_t *data);
+
 /*
  * Copy and join array of data into a single list.
  * IN data - array of data objects (list type only) to copy/merge into a single
@@ -496,6 +512,31 @@ extern data_t *data_list_join(const data_t **data, bool flatten_lists);
  */
 extern size_t data_get_list_length(const data_t *data);
 
+/*
+ * Get last entity in data list
+ * IN data data list to get last entry
+ * RET last entity in data list or NULL (empty list)
+ */
+extern data_t *data_get_list_last(data_t *data);
+
+/*
+ * Split up string using token and append values to dst
+ * IN dst - data list to append with values
+ * IN src - string to split by token
+ * IN token - token to split src with
+ * RET SLURM_SUCCESS or error
+ */
+extern int data_list_split_str(data_t *dst, const char *src, const char *token);
+
+/*
+ * Create string by joining strings in list with token
+ * IN dst - ptr to string to populate (caller must xfree())
+ * IN src - data list to join
+ * IN token - token to join src with
+ * RET SLURM_SUCCESS or error
+ */
+extern int data_list_join_str(char **dst, const data_t *src, const char *token);
+
 /*
  * Get data entry with given key (from constant data).
  * IN data constant data object to find entity with given key string
@@ -512,6 +553,29 @@ extern const data_t *data_key_get_const(const data_t *data, const char *key);
  */
 extern data_t *data_key_get(data_t *data, const char *key);
 
+/*
+ * Use match() function ptr find a specific matching value in dictionary
+ * IN data - data object to find entity with given key string
+ * IN match - strict weak ordering function that compares two data to arg
+ *   returning true if the first precedes the second.
+ * IN needle - match against needle value
+ * NOTE: the dictionaries are currently unordered but this is being implemented
+ *   as strict weak order to allow log(n) searching in the future.
+ * RET ptr to matching data or NULL if match not found
+ */
+extern data_t *data_dict_find_first(
+	data_t *data,
+	bool (*match)(const char *key, data_t *data, void *needle),
+	void *needle);
+
+/*
+ * Get data entry with given integer key
+ * IN data data object to find entity with given key string
+ * IN key string of key to find
+ * RET ptr to data or NULL if it doesn't exist.
+ */
+extern data_t *data_key_get_int(data_t *data, int64_t key);
+
 /*
  * Create data entry with given key.
  * Use this as a quick way to add or update a given key.
@@ -615,55 +679,4 @@ extern int data_retrieve_dict_path_int(const data_t *data, const char *path,
  */
 extern data_t *data_copy(data_t *dest, const data_t *src);
 
-typedef enum {
-	DATA_SER_FLAGS_NONE = 0, /* defaults to compact currently */
-	DATA_SER_FLAGS_COMPACT = 1 << 1,
-	DATA_SER_FLAGS_PRETTY = 1 << 2,
-} data_serializer_flags_t;
-
-/*
- * Define common MIME types to make it easier for serializer callers.
- *
- * WARNING: There is no guarantee that plugins for these types
- * will be loaded at any given time.
- */
-#define MIME_TYPE_YAML "application/x-yaml"
-#define MIME_TYPE_YAML_PLUGIN "serializer/yaml"
-#define MIME_TYPE_JSON "application/json"
-#define MIME_TYPE_JSON_PLUGIN "serializer/json"
-#define MIME_TYPE_URL_ENCODED "application/x-www-form-urlencoded"
-#define MIME_TYPE_URL_ENCODED_PLUGIN "serializer/url-encoded"
-
-/*
- * Serialize data in src into string dest
- * IN/OUT dest - ptr to NULL string ptr to set with output data.
- * 	caller must xfree(dest) if set.
- * IN src - populated data ptr to serialize
- * IN mime_type - serialize data into the given mime_type
- * IN flags - optional flags to specify to serilzier to change presentation of
- * 	data
- * RET SLURM_SUCCESS or error
- */
-extern int data_g_serialize(char **dest, const data_t *src,
-			    const char *mime_type,
-			    data_serializer_flags_t flags);
-
-/*
- * Deserialize string in src into data dest
- * IN/OUT dest - ptr to NULL data ptr to set with output data.
- * 	caller must FREE_NULL_DATA(dest) if set.
- * IN src - string to deserialize
- * IN length - number of bytes in src
- * IN mime_type - deserialize data using given mime_type
- * RET SLURM_SUCCESS or error
- */
-extern int data_g_deserialize(data_t **dest, const char *src, size_t length,
-			      const char *mime_type);
-
-/*
- * Check if there is a plugin loaded that can handle the requested mime type
- * RET ptr to best matching mime type or NULL if none can match
- */
-extern const char *data_resolve_mime_type(const char *mime_type);
-
 #endif /* _DATA_H */
