@@ -127,7 +127,7 @@ int io_hdr_read_fd(int fd, io_hdr_t *hdr)
 
 fail:
 	debug3("Leaving %s", __func__);
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return n;
 }
 
@@ -139,8 +139,7 @@ extern int io_init_msg_validate(io_init_msg_t *msg, const char *sig,
 	debug3("  msg->version = %x", msg->version);
 	debug3("  msg->nodeid = %u", msg->nodeid);
 
-	if (msg->version == IO_PROTOCOL_VERSION ||
-	    msg->version < SLURM_MIN_PROTOCOL_VERSION) {
+	if (msg->version < SLURM_MIN_PROTOCOL_VERSION) {
 		error("Invalid IO init header version");
 		return SLURM_ERROR;
 	}
@@ -158,7 +157,7 @@ extern int io_init_msg_validate(io_init_msg_t *msg, const char *sig,
 
 static int io_init_msg_pack(io_init_msg_t *hdr, buf_t *buffer)
 {
-	if (hdr->version == SLURM_PROTOCOL_VERSION) {
+	if (hdr->version >= SLURM_MIN_PROTOCOL_VERSION) {
 		uint32_t top_offset, tail_offset;
 		uint32_t len = 0;
 
@@ -176,18 +175,6 @@ static int io_init_msg_pack(io_init_msg_t *hdr, buf_t *buffer)
 		set_buf_offset(buffer, top_offset);
 		pack32(len, buffer);
 		set_buf_offset(buffer, tail_offset);
-	} else if (hdr->version == IO_PROTOCOL_VERSION) {
-		pack16(hdr->version, buffer);
-		pack32(hdr->nodeid, buffer);
-		pack32(hdr->stdout_objs, buffer);
-		pack32(hdr->stderr_objs, buffer);
-		if (hdr->io_key_len >= SLURM_IO_KEY_SIZE) {
-			packmem(hdr->io_key, SLURM_IO_KEY_SIZE, buffer);
-		} else {
-			char tmp_key[SLURM_IO_KEY_SIZE] = { 0 };
-			memcpy(tmp_key, hdr->io_key, hdr->io_key_len);
-			packmem(tmp_key, SLURM_IO_KEY_SIZE, buffer);
-		}
 	} else {
 		error("Invalid IO init header version");
 		return SLURM_ERROR;
@@ -202,8 +189,7 @@ static int io_init_msg_unpack(io_init_msg_t *hdr, buf_t *buffer)
 	/* If this function changes, io_init_msg_packed_size must change. */
 
 	safe_unpack16(&hdr->version, buffer);
-	if (hdr->version != IO_PROTOCOL_VERSION &&
-	    hdr->version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (hdr->version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&hdr->nodeid, buffer);
 		safe_unpack32(&hdr->stdout_objs, buffer);
 		safe_unpack32(&hdr->stderr_objs, buffer);
@@ -236,7 +222,7 @@ io_init_msg_write_to_fd(int fd, io_init_msg_t *msg)
 	rc = SLURM_SUCCESS;
 
 rwfail:
-	free_buf(buf);
+	FREE_NULL_BUFFER(buf);
 	debug2("%s: leaving", __func__);
 	return rc;
 }
@@ -263,12 +249,12 @@ extern int io_init_msg_read_from_fd(int fd, io_init_msg_t *msg)
 	if ((rc = io_init_msg_unpack(msg, buf)))
 		error("%s: io_init_msg_unpack failed: rc=%d", __func__, rc);
 
-	free_buf(buf);
+	FREE_NULL_BUFFER(buf);
 	debug2("Leaving %s", __func__);
 	return rc;
 
 rwfail:
-	free_buf(buf);
+	FREE_NULL_BUFFER(buf);
 	error("%s: reading slurm_io_init_msg failed: %m",__func__);
 	return SLURM_ERROR;
 }
