@@ -49,6 +49,7 @@
 #include "src/common/pack.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
 #include "src/slurmctld/slurmctld.h"
 
 
@@ -74,8 +75,8 @@ extern job_resources_t *create_job_resources(void)
  */
 extern int build_job_resources(job_resources_t *job_resrcs)
 {
-	int i, bitmap_len;
 	int core_cnt = 0, sock_inx = -1;
+	node_record_t *node_ptr;
 
 	if (job_resrcs->node_bitmap == NULL) {
 		error("build_job_resources: node_bitmap is NULL");
@@ -92,12 +93,8 @@ extern int build_job_resources(job_resources_t *job_resrcs)
 	job_resrcs->sock_core_rep_count = xcalloc(job_resrcs->nhosts,
 						  sizeof(uint32_t));
 
-	bitmap_len = bit_size(job_resrcs->node_bitmap);
-	for (i=0; i<bitmap_len; i++) {
-		if (!bit_test(job_resrcs->node_bitmap, i))
-			continue;
-		node_record_t *node_ptr = node_record_table_ptr[i];
-
+	for (int i = 0;
+	     (node_ptr = next_node_bitmap(job_resrcs->node_bitmap, &i)); i++) {
 		if ((sock_inx < 0) ||
 		    (node_ptr->tot_sockets !=
 		     job_resrcs->sockets_per_node[sock_inx]) ||
@@ -128,9 +125,9 @@ extern int build_job_resources(job_resources_t *job_resrcs)
  * Return total CPU count or -1 on error */
 extern int build_job_resources_cpu_array(job_resources_t *job_resrcs_ptr)
 {
-	int cpu_count = 0, i;
+	int cpu_count = 0;
 	uint32_t last_cpu_cnt = NO_VAL;
-	int j, i_first, i_last, node_cpu_count;
+	int node_cpu_count;
 
 	if (job_resrcs_ptr->nhosts == 0)
 		return cpu_count;	/* no work to do */
@@ -148,16 +145,8 @@ extern int build_job_resources_cpu_array(job_resources_t *job_resrcs_ptr)
 	job_resrcs_ptr->cpu_array_value = xcalloc(job_resrcs_ptr->nhosts,
 						  sizeof(uint16_t));
 
-	i_first = bit_ffs(job_resrcs_ptr->node_bitmap);
-	if (i_first != -1)
-		i_last = bit_fls(job_resrcs_ptr->node_bitmap);
-	else
-		i_last = -2;
-
-	for (i = i_first, j = 0; i <= i_last; i++) {
-		if (!bit_test(job_resrcs_ptr->node_bitmap, i))
-			continue;
-
+	for (int i = 0, j = 0;
+	     next_node_bitmap(job_resrcs_ptr->node_bitmap, &i); i++) {
 		/*
 		 * This needs to be the threads per core count to handle
 		 * allocations correctly.
@@ -184,54 +173,6 @@ extern int build_job_resources_cpu_array(job_resources_t *job_resrcs_ptr)
 	return cpu_count;
 }
 
-/* Rebuild cpus array based upon the values of nhosts, cpu_array_value and
- * cpu_array_reps in an existing data structure
- * Return total CPU count or -1 on error */
-extern int build_job_resources_cpus_array(job_resources_t *job_resrcs_ptr)
-{
-	int cpu_count = 0, cpu_inx, i, j;
-
-	if (job_resrcs_ptr->nhosts == 0)
-		return cpu_count;	/* no work to do */
-	if (job_resrcs_ptr->cpu_array_cnt == 0) {
-		error("build_job_resources_cpus_array: cpu_array_cnt==0");
-		return -1;
-	}
-	if (job_resrcs_ptr->cpu_array_value == NULL) {
-		error("build_job_resources_cpus_array: cpu_array_value==NULL");
-		return -1;
-	}
-	if (job_resrcs_ptr->cpu_array_reps == NULL) {
-		error("build_job_resources_cpus_array: cpu_array_reps==NULL");
-		return -1;
-	}
-
-	/* clear vestigial data and create new arrays of max size */
-	xfree(job_resrcs_ptr->cpus);
-	job_resrcs_ptr->cpus = xcalloc(job_resrcs_ptr->nhosts,
-				       sizeof(uint16_t));
-
-	cpu_inx = 0;
-	for (i=0; i<job_resrcs_ptr->cpu_array_cnt; i++) {
-		for (j=0; j<job_resrcs_ptr->cpu_array_reps[i]; j++) {
-			if (cpu_inx >= job_resrcs_ptr->nhosts) {
-				error("build_job_resources_cpus_array: "
-				      "cpu_array is too long");
-				return -1;
-			}
-			cpu_count += job_resrcs_ptr->cpus[i];
-			job_resrcs_ptr->cpus[cpu_inx++] =
-				job_resrcs_ptr->cpus[i];
-		}
-	}
-	if (cpu_inx < job_resrcs_ptr->nhosts) {
-		error("build_job_resources_cpus_array: "
-		      "cpu_array is incomplete");
-		return -1;
-	}
-	return cpu_count;
-}
-
 /* Reset the node_bitmap in a job_resources data structure
  * This is needed after a restart/reconfiguration since nodes can
  * be added or removed from the system resulting in changing in
@@ -266,12 +207,11 @@ extern int reset_node_bitmap(void *void_job_ptr)
 	return SLURM_SUCCESS;
 }
 
-extern int valid_job_resources(job_resources_t *job_resrcs,
-			       node_record_t **node_rec_table)
+extern int valid_job_resources(job_resources_t *job_resrcs)
 {
-	int i, bitmap_len;
 	int sock_inx = 0, sock_cnt = 0;
 	int total_job_cores, total_node_cores;
+	node_record_t *node_ptr;
 
 	if (job_resrcs->node_bitmap == NULL) {
 		error("valid_job_resources: node_bitmap is NULL");
@@ -284,11 +224,8 @@ extern int valid_job_resources(job_resources_t *job_resrcs,
 		return SLURM_ERROR;
 	}
 
-	bitmap_len = bit_size(job_resrcs->node_bitmap);
-	for (i=0; i<bitmap_len; i++) {
-		if (!bit_test(job_resrcs->node_bitmap, i))
-			continue;
-
+	for (int i = 0;
+	     (node_ptr = next_node_bitmap(job_resrcs->node_bitmap, &i)); i++) {
 		if (sock_cnt >= job_resrcs->sock_core_rep_count[sock_inx]) {
 			sock_inx++;
 			sock_cnt = 0;
@@ -297,13 +234,13 @@ extern int valid_job_resources(job_resources_t *job_resrcs,
 		 * but the socket/NUMA count can change on reboot */
 		total_job_cores = job_resrcs->sockets_per_node[sock_inx] *
 				  job_resrcs->cores_per_socket[sock_inx];
-		total_node_cores = node_rec_table[i]->tot_cores;
+		total_node_cores = node_ptr->tot_cores;
 		if (total_job_cores != total_node_cores) {
 			error("valid_job_resources: %s sockets:%u,%u, cores %u,%u",
-			      node_rec_table[i]->name,
-			      node_rec_table[i]->tot_sockets,
+			      node_ptr->name,
+			      node_ptr->tot_sockets,
 			      job_resrcs->sockets_per_node[sock_inx],
-			      node_rec_table[i]->cores,
+			      node_ptr->cores,
 			      job_resrcs->cores_per_socket[sock_inx]);
 			return SLURM_ERROR;
 		}
@@ -319,6 +256,7 @@ extern job_resources_t *copy_job_resources(job_resources_t *job_resrcs_ptr)
 
 	xassert(job_resrcs_ptr);
 	new_layout->nhosts = job_resrcs_ptr->nhosts;
+	new_layout->nodes = xstrdup(job_resrcs_ptr->nodes);
 	new_layout->ncpus = job_resrcs_ptr->ncpus;
 	new_layout->node_req = job_resrcs_ptr->node_req;
 	new_layout->whole_node = job_resrcs_ptr->whole_node;
@@ -630,7 +568,7 @@ extern void pack_job_resources(job_resources_t *job_resrcs_ptr, buf_t *buffer,
 		pack_bit_str_hex(job_resrcs_ptr->core_bitmap, buffer);
 		pack_bit_str_hex(job_resrcs_ptr->core_bitmap_used,
 				 buffer);
-	} else if (protocol_version >= SLURM_21_08_PROTOCOL_VERSION) {
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		if (job_resrcs_ptr == NULL) {
 			uint32_t empty = NO_VAL;
 			pack32(empty, buffer);
@@ -687,80 +625,6 @@ extern void pack_job_resources(job_resources_t *job_resrcs_ptr, buf_t *buffer,
 		xassert(job_resrcs_ptr->sock_core_rep_count);
 		xassert(job_resrcs_ptr->sockets_per_node);
 
-		for (i=0; i < job_resrcs_ptr->nhosts; i++) {
-			sock_recs += job_resrcs_ptr->
-				     sock_core_rep_count[i];
-			if (sock_recs >= job_resrcs_ptr->nhosts)
-				break;
-		}
-		i++;
-		pack16_array(job_resrcs_ptr->sockets_per_node,
-			     (uint32_t) i, buffer);
-		pack16_array(job_resrcs_ptr->cores_per_socket,
-			     (uint32_t) i, buffer);
-		pack32_array(job_resrcs_ptr->sock_core_rep_count,
-			     (uint32_t) i, buffer);
-
-		xassert(job_resrcs_ptr->core_bitmap);
-		xassert(job_resrcs_ptr->core_bitmap_used);
-		pack_bit_str_hex(job_resrcs_ptr->core_bitmap, buffer);
-		pack_bit_str_hex(job_resrcs_ptr->core_bitmap_used,
-				 buffer);
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		if (job_resrcs_ptr == NULL) {
-			uint32_t empty = NO_VAL;
-			pack32(empty, buffer);
-			return;
-		}
-
-		pack32(job_resrcs_ptr->nhosts, buffer);
-		pack32(job_resrcs_ptr->ncpus, buffer);
-		pack32(job_resrcs_ptr->node_req, buffer);
-		packstr(job_resrcs_ptr->nodes, buffer);
-		pack8(job_resrcs_ptr->whole_node, buffer);
-
-		if (job_resrcs_ptr->cpu_array_reps)
-			pack32_array(job_resrcs_ptr->cpu_array_reps,
-				     job_resrcs_ptr->cpu_array_cnt, buffer);
-		else
-			pack32_array(job_resrcs_ptr->cpu_array_reps, 0, buffer);
-
-		if (job_resrcs_ptr->cpu_array_value)
-			pack16_array(job_resrcs_ptr->cpu_array_value,
-				     job_resrcs_ptr->cpu_array_cnt, buffer);
-		else
-			pack16_array(job_resrcs_ptr->cpu_array_value,
-				     0, buffer);
-
-		if (job_resrcs_ptr->cpus)
-			pack16_array(job_resrcs_ptr->cpus,
-				     job_resrcs_ptr->nhosts, buffer);
-		else
-			pack16_array(job_resrcs_ptr->cpus, 0, buffer);
-
-		if (job_resrcs_ptr->cpus_used)
-			pack16_array(job_resrcs_ptr->cpus_used,
-				     job_resrcs_ptr->nhosts, buffer);
-		else
-			pack16_array(job_resrcs_ptr->cpus_used, 0, buffer);
-
-		if (job_resrcs_ptr->memory_allocated)
-			pack64_array(job_resrcs_ptr->memory_allocated,
-				     job_resrcs_ptr->nhosts, buffer);
-		else
-			pack64_array(job_resrcs_ptr->memory_allocated,
-				     0, buffer);
-
-		if (job_resrcs_ptr->memory_used)
-			pack64_array(job_resrcs_ptr->memory_used,
-				     job_resrcs_ptr->nhosts, buffer);
-		else
-			pack64_array(job_resrcs_ptr->memory_used, 0, buffer);
-
-		xassert(job_resrcs_ptr->cores_per_socket);
-		xassert(job_resrcs_ptr->sock_core_rep_count);
-		xassert(job_resrcs_ptr->sockets_per_node);
-
 		for (i=0; i < job_resrcs_ptr->nhosts; i++) {
 			sock_recs += job_resrcs_ptr->
 				     sock_core_rep_count[i];
@@ -857,7 +721,7 @@ extern int unpack_job_resources(job_resources_t **job_resrcs_pptr,
 		unpack_bit_str_hex(&job_resrcs->core_bitmap, buffer);
 		unpack_bit_str_hex(&job_resrcs->core_bitmap_used,
 				   buffer);
-	} else if (protocol_version >= SLURM_21_08_PROTOCOL_VERSION) {
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&empty, buffer);
 		if (empty == NO_VAL) {
 			*job_resrcs_pptr = NULL;
@@ -879,67 +743,6 @@ extern int unpack_job_resources(job_resources_t **job_resrcs_pptr,
 			xfree(job_resrcs->cpu_array_reps);
 		job_resrcs->cpu_array_cnt = tmp32;
 
-		safe_unpack16_array(&job_resrcs->cpu_array_value,
-				    &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->cpu_array_value);
-
-		if (tmp32 != job_resrcs->cpu_array_cnt)
-			goto unpack_error;
-
-		safe_unpack16_array(&job_resrcs->cpus, &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->cpus);
-		if (tmp32 != job_resrcs->nhosts)
-			goto unpack_error;
-		safe_unpack16_array(&job_resrcs->cpus_used, &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->cpus_used);
-
-		safe_unpack64_array(&job_resrcs->memory_allocated,
-				    &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->memory_allocated);
-		safe_unpack64_array(&job_resrcs->memory_used, &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->memory_used);
-
-		safe_unpack16_array(&job_resrcs->sockets_per_node,
-				    &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->sockets_per_node);
-		safe_unpack16_array(&job_resrcs->cores_per_socket,
-				    &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->cores_per_socket);
-		safe_unpack32_array(&job_resrcs->sock_core_rep_count,
-				    &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->sock_core_rep_count);
-
-		unpack_bit_str_hex(&job_resrcs->core_bitmap, buffer);
-		unpack_bit_str_hex(&job_resrcs->core_bitmap_used,
-				   buffer);
-	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		safe_unpack32(&empty, buffer);
-		if (empty == NO_VAL) {
-			*job_resrcs_pptr = NULL;
-			return SLURM_SUCCESS;
-		}
-
-		job_resrcs = xmalloc(sizeof(struct job_resources));
-		job_resrcs->nhosts = empty;
-		safe_unpack32(&job_resrcs->ncpus, buffer);
-		safe_unpack32(&job_resrcs->node_req, buffer);
-		safe_unpackstr_xmalloc(&job_resrcs->nodes, &tmp32, buffer);
-		safe_unpack8(&job_resrcs->whole_node, buffer);
-
-		safe_unpack32_array(&job_resrcs->cpu_array_reps,
-				    &tmp32, buffer);
-		if (tmp32 == 0)
-			xfree(job_resrcs->cpu_array_reps);
-		job_resrcs->cpu_array_cnt = tmp32;
-
 		safe_unpack16_array(&job_resrcs->cpu_array_value,
 				    &tmp32, buffer);
 		if (tmp32 == 0)
@@ -1141,7 +944,7 @@ extern int job_resources_and(job_resources_t *job_resrcs1_ptr,
 			     job_resources_t *job_resrcs2_ptr)
 {
 	int i, i_first, i_last, j;
-	int node_cnt, node_inx1 = -1, node_inx2 = -1;
+	int node_cnt;
 	int sock_core_cnt1 = 0, sock_core_cnt2 = 0;
 	int so_co_off1 = 0, so_co_off2 = 0;
 	int core_cnt, core_cnt1, core_cnt2;;
@@ -1180,14 +983,10 @@ extern int job_resources_and(job_resources_t *job_resrcs1_ptr,
 		i_last = -2;
 	for (i = i_first; i <= i_last; i++) {
 		bool match1 = false, match2 = false;
-		if (bit_test(job_resrcs1_ptr->node_bitmap, i)) {
-			node_inx1++;
+		if (bit_test(job_resrcs1_ptr->node_bitmap, i))
 			match1 = true;
-		}
-		if (bit_test(job_resrcs2_ptr->node_bitmap, i)) {
-			node_inx2++;
+		if (bit_test(job_resrcs2_ptr->node_bitmap, i))
 			match2 = true;
-		}
 		if (!match1 && !match2)	/* Unused node */
 			continue;
 		if (match1 && match2) {	/* Merge (AND) core_bitmaps */
@@ -1269,7 +1068,7 @@ extern int job_resources_or(job_resources_t *job_resrcs1_ptr,
 {
 	job_resources_t *job_resrcs_new;
 	int i, i_first, i_last, j;
-	int node_cnt, node_inx = -1, node_inx1 = -1, node_inx2 = -1;
+	int node_cnt, node_inx = -1;
 	int sock_core_cnt1 = 0, sock_core_cnt2 = 0;
 	int so_co_off1 = 0, so_co_off2 = 0;
 	int core_cnt, core_cnt1, core_cnt2;
@@ -1318,14 +1117,10 @@ extern int job_resources_or(job_resources_t *job_resrcs1_ptr,
 		i_last = -2;
 	for (i = i_first; i <= i_last; i++) {
 		bool match1 = false, match2 = false;
-		if (bit_test(job_resrcs1_ptr->node_bitmap, i)) {
-			node_inx1++;
+		if (bit_test(job_resrcs1_ptr->node_bitmap, i))
 			match1 = true;
-		}
-		if (bit_test(job_resrcs2_ptr->node_bitmap, i)) {
-			node_inx2++;
+		if (bit_test(job_resrcs2_ptr->node_bitmap, i))
 			match2 = true;
-		}
 		if (!match1 && !match2)	/* Unused node */
 			continue;
 		bit_set(job_resrcs_new->node_bitmap, i);
@@ -1421,9 +1216,9 @@ extern int job_resources_or(job_resources_t *job_resrcs1_ptr,
 
 	/* Update data structure fields as needed */
 	job_resrcs1_ptr->nhosts = node_inx + 1;
-	bit_free(job_resrcs1_ptr->core_bitmap);
+	FREE_NULL_BITMAP(job_resrcs1_ptr->core_bitmap);
 	job_resrcs1_ptr->core_bitmap = job_resrcs_new->core_bitmap;
-	bit_free(job_resrcs1_ptr->node_bitmap);
+	FREE_NULL_BITMAP(job_resrcs1_ptr->node_bitmap);
 	job_resrcs1_ptr->node_bitmap = job_resrcs_new->node_bitmap;
 	xfree(job_resrcs1_ptr->cores_per_socket);
 	job_resrcs1_ptr->cores_per_socket = job_resrcs_new->cores_per_socket;
@@ -1536,7 +1331,7 @@ extern int clear_job_resources_node(job_resources_t *job_resrcs_ptr,
 /* Completely remove specified node from job resources structure */
 extern int extract_job_resources_node(job_resources_t *job, uint32_t node_id)
 {
-	int i, i_first, i_last, n;
+	int i, n;
 	int bit_inx = 0, core_cnt = 0, host_cnt, len, node_inx = node_id;
 
 	xassert(job);
@@ -1594,14 +1389,7 @@ extern int extract_job_resources_node(job_resources_t *job, uint32_t node_id)
 		bit_realloc(job->core_bitmap_used, len - core_cnt);
 
 	/* Shift cpus, cpus_used, memory_allocated, and memory_used arrays */
-	i_first = bit_ffs(job->node_bitmap);
-	if (i_first >= 0)
-		i_last = bit_fls(job->node_bitmap);
-	else
-		i_last = i_first - 1;
-	for (i = i_first, n = -1; i <= i_last; i++) {
-		if (!bit_test(job->node_bitmap, i))
-			continue;
+	for (i = 0, n = -1; next_node_bitmap(job->node_bitmap, &i); i++) {
 		if (++n == node_id) {
 			bit_clear(job->node_bitmap, i);
 			break;
@@ -1761,28 +1549,24 @@ extern int job_fits_into_cores(job_resources_t *job_resrcs_ptr,
 			       const uint16_t *bits_per_node)
 {
 	int full_node_inx = 0, full_bit_inx  = 0, job_bit_inx  = 0, i;
-	int job_node_cnt;
 
 	if (!full_bitmap)
 		return 1;
 
-	job_node_cnt = bit_set_count(job_resrcs_ptr->node_bitmap);
-	for (full_node_inx = bit_ffs(job_resrcs_ptr->node_bitmap);
-	     job_node_cnt > 0; full_node_inx++) {
-		if (bit_test(job_resrcs_ptr->node_bitmap, full_node_inx)) {
-			full_bit_inx = cr_node_cores_offset[full_node_inx];
-			for (i = 0; i < bits_per_node[full_node_inx]; i++) {
-				if (!bit_test(full_bitmap, full_bit_inx + i))
-					continue;
-				if ((job_resrcs_ptr->whole_node == 1) ||
-				    bit_test(job_resrcs_ptr->core_bitmap,
-					     job_bit_inx + i)) {
-					return 0;
-				}
+	for (full_node_inx = 0;
+	     next_node_bitmap(job_resrcs_ptr->node_bitmap, &full_node_inx);
+	     full_node_inx++) {
+		full_bit_inx = cr_node_cores_offset[full_node_inx];
+		for (i = 0; i < bits_per_node[full_node_inx]; i++) {
+			if (!bit_test(full_bitmap, full_bit_inx + i))
+				continue;
+			if ((job_resrcs_ptr->whole_node == 1) ||
+				bit_test(job_resrcs_ptr->core_bitmap,
+					job_bit_inx + i)) {
+				return 0;
 			}
-			job_bit_inx += bits_per_node[full_node_inx];
-			job_node_cnt --;
 		}
+		job_bit_inx += bits_per_node[full_node_inx];
 	}
 	return 1;
 }
@@ -1798,7 +1582,7 @@ extern void add_job_to_cores(job_resources_t *job_resrcs_ptr,
 			     bitstr_t **full_core_bitmap,
 			     const uint16_t *bits_per_node)
 {
-	int full_node_inx = 0, job_node_cnt;
+	int full_node_inx = 0;
 	int job_bit_inx  = 0, full_bit_inx  = 0, i;
 
 	if (!job_resrcs_ptr->core_bitmap)
@@ -1807,69 +1591,23 @@ extern void add_job_to_cores(job_resources_t *job_resrcs_ptr,
 	/* add the job to the row_bitmap */
 	if (*full_core_bitmap == NULL) {
 		uint32_t size = 0;
-		for (i = 0; i < node_record_count; i++)
+		for (i = 0; next_node(&i); i++)
 			size += bits_per_node[i];
 		*full_core_bitmap = bit_alloc(size);
 	}
 
-	job_node_cnt = bit_set_count(job_resrcs_ptr->node_bitmap);
-	for (full_node_inx = bit_ffs(job_resrcs_ptr->node_bitmap);
-	     job_node_cnt > 0; full_node_inx++) {
-		if (bit_test(job_resrcs_ptr->node_bitmap, full_node_inx)) {
-			full_bit_inx = cr_node_cores_offset[full_node_inx];
-			for (i = 0; i < bits_per_node[full_node_inx]; i++) {
-				if ((job_resrcs_ptr->whole_node != 1) &&
-				    !bit_test(job_resrcs_ptr->core_bitmap,
-					      job_bit_inx + i))
-					continue;
-				bit_set(*full_core_bitmap, full_bit_inx + i);
-			}
-			job_bit_inx += bits_per_node[full_node_inx];
-			job_node_cnt --;
-		}
-	}
-}
-
-/*
- * Remove job from full-length core_bitmap
- * IN job_resrcs_ptr - resources allocated to a job
- * IN/OUT full_bitmap - bitmap of available CPUs, allocate as needed
- * IN bits_per_node - bits per node in the full_bitmap
- * RET 1 on success, 0 otherwise
- */
-extern void remove_job_from_cores(job_resources_t *job_resrcs_ptr,
-				  bitstr_t **full_core_bitmap,
-				  const uint16_t *bits_per_node)
-{
-	int full_node_inx = 0, job_node_cnt;
-	int job_bit_inx  = 0, full_bit_inx  = 0, i;
-
-	if (!job_resrcs_ptr->core_bitmap)
-		return;
-
-	/* add the job to the row_bitmap */
-	if (*full_core_bitmap == NULL) {
-		uint32_t size = 0;
-		for (i = 0; i < node_record_count; i++)
-			size += bits_per_node[i];
-		*full_core_bitmap = bit_alloc(size);
-	}
-
-	job_node_cnt = bit_set_count(job_resrcs_ptr->node_bitmap);
-	for (full_node_inx = bit_ffs(job_resrcs_ptr->node_bitmap);
-	     job_node_cnt > 0; full_node_inx++) {
-		if (bit_test(job_resrcs_ptr->node_bitmap, full_node_inx)) {
-			full_bit_inx = cr_node_cores_offset[full_node_inx];
-			for (i = 0; i < bits_per_node[full_node_inx]; i++) {
-				if ((job_resrcs_ptr->whole_node != 1) &&
-				    !bit_test(job_resrcs_ptr->core_bitmap,
-					      job_bit_inx + i))
-					continue;
-				bit_clear(*full_core_bitmap, full_bit_inx + i);
-			}
-			job_bit_inx += bits_per_node[full_node_inx];
-			job_node_cnt --;
+	for (full_node_inx = 0;
+	     next_node_bitmap(job_resrcs_ptr->node_bitmap, &full_node_inx);
+	     full_node_inx++) {
+		full_bit_inx = cr_node_cores_offset[full_node_inx];
+		for (i = 0; i < bits_per_node[full_node_inx]; i++) {
+			if ((job_resrcs_ptr->whole_node != 1) &&
+			    !bit_test(job_resrcs_ptr->core_bitmap,
+				      job_bit_inx + i))
+				continue;
+			bit_set(*full_core_bitmap, full_bit_inx + i);
 		}
+		job_bit_inx += bits_per_node[full_node_inx];
 	}
 }
 
@@ -1880,7 +1618,7 @@ extern void remove_job_from_cores(job_resources_t *job_resrcs_ptr,
 extern int job_resources_node_inx_to_cpu_inx(job_resources_t *job_resrcs_ptr,
 					     int node_inx)
 {
-	int first_inx, i, node_offset;
+	int node_offset;
 
 	/* Test for error cases */
 	if (!job_resrcs_ptr || !job_resrcs_ptr->node_bitmap) {
@@ -1907,12 +1645,8 @@ extern int job_resources_node_inx_to_cpu_inx(job_resources_t *job_resrcs_ptr,
 	if (job_resrcs_ptr->nhosts == 1)
 		return 0;
 
-	/* Scan bitmap, convert node_inx to node_cnt within job's allocation */
-	first_inx = bit_ffs(job_resrcs_ptr->node_bitmap);
-	for (i = first_inx, node_offset = -1; i <= node_inx; i++) {
-		if (bit_test(job_resrcs_ptr->node_bitmap, i))
-			node_offset++;
-	}
+	node_offset = bit_set_count_range(job_resrcs_ptr->node_bitmap, 0,
+					  node_inx);
 
 	if (node_offset >= job_resrcs_ptr->nhosts) {
 		error("%s: Found %d of %d nodes", __func__,
