@@ -58,6 +58,7 @@
 ** for details.
 */
 strong_alias(parse_time, slurm_parse_time);
+strong_alias(parse_time_make_str_utc, slurm_parse_time_make_str_utc);
 //slurm_make_time_str is already exported
 strong_alias(time_str2mins, slurm_time_str2mins);
 strong_alias(time_str2secs, slurm_time_str2secs);
@@ -646,6 +647,43 @@ static char *_relative_date_fmt(const struct tm *when)
 	return "%a %H:%M";			/* near distance */
 }
 
+static void _make_time_str_internal(time_t *time, bool utc, char *string,
+				    int size)
+{
+	struct tm time_tm;
+
+	if (utc)
+		gmtime_r(time, &time_tm);
+	else
+		localtime_r(time, &time_tm);
+	if ((*time == (time_t) 0) || (*time == (time_t) INFINITE)) {
+		snprintf(string, size, "Unknown");
+	} else if (*time == (time_t) NO_VAL) {
+		snprintf(string, size, "None");
+	} else {
+		static char fmt_buf[32];
+		static const char *display_fmt = "%FT%T";
+
+		if (!utc) {
+			char *fmt = getenv("SLURM_TIME_FORMAT");
+
+			if ((!fmt) || (!*fmt) || (!xstrcmp(fmt, "standard"))) {
+				;
+			} else if (xstrcmp(fmt, "relative") == 0) {
+				display_fmt = _relative_date_fmt(&time_tm);
+			} else if ((strchr(fmt, '%')  == NULL) ||
+				   (strlen(fmt) >= sizeof(fmt_buf))) {
+				error("invalid SLURM_TIME_FORMAT = '%s'", fmt);
+			} else {
+				strlcpy(fmt_buf, fmt, sizeof(fmt_buf));
+				display_fmt = fmt_buf;
+			}
+		}
+
+		slurm_strftime(string, size, display_fmt, &time_tm);
+	}
+}
+
 /*
  * slurm_make_time_str - convert time_t to formatted string for user output
  *
@@ -661,39 +699,19 @@ static char *_relative_date_fmt(const struct tm *when)
 extern void
 slurm_make_time_str (time_t *time, char *string, int size)
 {
-	struct tm time_tm;
+	_make_time_str_internal(time, false, string, size);
+}
 
-	localtime_r(time, &time_tm);
-	if ((*time == (time_t) 0) || (*time == (time_t) INFINITE)) {
-		snprintf(string, size, "Unknown");
-	} else if (*time == (time_t) NO_VAL) {
-		snprintf(string, size, "None");
-	} else {
-		static char fmt_buf[32];
-		static const char *display_fmt;
-		static bool use_relative_format;
-
-		if (!display_fmt) {
-			char *fmt = getenv("SLURM_TIME_FORMAT");
-			display_fmt = "%FT%T";
-
-			if ((!fmt) || (!*fmt) || (!xstrcmp(fmt, "standard"))) {
-				;
-			} else if (xstrcmp(fmt, "relative") == 0) {
-				use_relative_format = true;
-			} else if ((strchr(fmt, '%')  == NULL) ||
-				   (strlen(fmt) >= sizeof(fmt_buf))) {
-				error("invalid SLURM_TIME_FORMAT = '%s'", fmt);
-			} else {
-				strlcpy(fmt_buf, fmt, sizeof(fmt_buf));
-				display_fmt = fmt_buf;
-			}
-		}
-		if (use_relative_format)
-			display_fmt = _relative_date_fmt(&time_tm);
-
-		slurm_strftime(string, size, display_fmt, &time_tm);
-	}
+/*
+ * Convert time_t to fixed "%FT%T" formatted string expressed in UTC.
+ *
+ * IN time - a timestamp
+ * OUT string - pointer user defined buffer
+ * IN size - length of string buffer (recommend 32 bytes)
+ */
+extern void parse_time_make_str_utc(time_t *time, char *string, int size)
+{
+	_make_time_str_internal(time, true, string, size);
 }
 
 /* Convert a string to an equivalent time value
