@@ -49,7 +49,7 @@
 #include "slurm/slurm_errno.h"
 
 #include "src/common/pack.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_protocol_common.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_protocol_util.h"
@@ -213,6 +213,11 @@ extern int slurm_init_msg_engine_ports(uint16_t *);
 
 /*
  * bind() and then listen() to any port in a given range of ports
+ *
+ * IN: s - socket
+ * IN: port - port number to attempt to bind
+ * IN: local - only bind to localhost if true
+ * OUT: port was bound successfully or -1 on failure
  */
 extern int sock_bind_listen_range(int s, uint16_t *range, bool local);
 
@@ -417,20 +422,30 @@ extern void slurm_pack_addr_array(slurm_addr_t *addr_array, uint32_t size_val,
 extern int slurm_unpack_addr_array(slurm_addr_t **addr_array_ptr,
 				   uint32_t *size_val, buf_t *buffer);
 
+/**********************************************************************\
+ * message packing routines
+\**********************************************************************/
+
+/*
+ * Pack message into buffers to be ready to send.
+ *
+ * IN msg - message to pack
+ * IN buffers - buffers to populate with packed message
+ * IN block_for_forwarding - call the forward_wait() which blocks until
+ *    forwarding
+ * RET SLURM_SUCCESS or error
+ */
+extern int slurm_buffers_pack_msg(slurm_msg_t *msg, msg_bufs_t *buffers,
+				  bool block_for_forwarding);
+
 /**********************************************************************\
  * simplified communication routines
  * They open a connection do work then close the connection all within
  * the function
 \**********************************************************************/
 
-/* slurm_send_msg
- * given the original request message this function sends a
- *	arbitrary message back to the client that made the request
- * IN request_msg	- slurm_msg the request msg
- * IN msg_type          - message type being returned
- * IN resp_msg		- the message being returned to the client
- */
-int slurm_send_msg(slurm_msg_t *msg, uint16_t msg_type, void *resp);
+extern void response_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
+			  uint16_t msg_type, void *data);
 
 /* slurm_send_rc_msg
  * given the original request message this function sends a
