@@ -155,7 +155,6 @@ try_again:
 			errno = 0;
 			goto end_it;
 		}
-		error("mysql_query failed: %d %s\n%s", errno, err_str, query);
 		if (errno == ER_LOCK_DEADLOCK) {
 			/*
 			 * Mysql detected a deadlock and we should retry
@@ -189,6 +188,7 @@ try_again:
 			      "You will need to call 'mysqladmin flush-hosts' "
 			      "to regain connectivity.");
 		}
+		error("mysql_query failed: %d %s\n%s", errno, err_str, query);
 		rc = SLURM_ERROR;
 	}
 end_it:
@@ -202,6 +202,79 @@ end_it:
 	return rc;
 }
 
+/*
+ * Determine if a database server upgrade has taken place and if so, check to
+ * see if the candidate table alteration query should be used to alter the table
+ * to its expected settings. Returns true if so.
+ *
+ * Background:
+ *
+ * From the MariaDB docs:
+ *  Before MariaDB 10.2.1, BLOB and TEXT columns could not be assigned a DEFAULT
+ *  value. This restriction was lifted in MariaDB 10.2.1.
+ *
+ * If a site begins using MariaDB >= 10.2.1 and is either using an existing
+ * Slurm database from an earlier version or has restored one from a dump from
+ * an earlier version or from any version of MySQL, some text/blob default
+ * values will need to be altered to avoid failures from subsequent queries from
+ * slurmdbd that set affected fields to DEFAULT (see bug#13606).
+ *
+ * Note that only one column from one table ('preempt' from qos_table) is
+ * checked to determine if an upgrade has taken place with the assumption that
+ * if its default value is not correct then the same is true for similar
+ * text/blob columns from other tables and they will also need to be altered.
+ *
+ * The qos_table has been chosen for this check because it is the last table
+ * with condition to be created. If that condition changes this should be
+ * re-evaluated.
+ */
+static bool _alter_table_after_upgrade(mysql_conn_t *mysql_conn,
+				       char *table_alter_query)
+{
+	static bool have_value = false, upgraded = false;
+	MYSQL_RES *result = NULL;
+	MYSQL_ROW row;
+
+	/* check to see if upgrade has happened */
+	if (!have_value) {
+		const char *info;
+		char *query;
+		/*
+		 * confirm MariaDB is being used to avoid any ambiguity with
+		 * MySQL versions
+		 */
+		info = mysql_get_server_info(mysql_conn->db_conn);
+		if (xstrcasestr(info, "mariadb") &&
+		    (mysql_get_server_version(mysql_conn->db_conn) >= 100201)) {
+			query = "show columns from `qos_table` like 'preempt'";
+			result = mysql_db_query_ret(mysql_conn, query, 0);
+			if (result) {
+				/*
+				 * row[4] holds the column's default value and
+				 * if it's NULL then it will need to be altered
+				 * and an upgrade is assumed
+				 */
+				if ((row = mysql_fetch_row(result)) &&
+				    !xstrcasecmp(row[1], "text") &&
+				    !row[4])
+					upgraded = true;
+				mysql_free_result(result);
+			}
+		}
+		have_value = true;
+	}
+
+	/*
+	 * If upgrade detected and the table alter query string contains an
+	 * emtpy string default then the query should be executed. The latter
+	 * check avoids unnecessary table alterations.
+	 */
+	if (upgraded && xstrcasestr(table_alter_query, "default ''"))
+		return true;
+
+	return false;
+}
+
 /* NOTE: Ensure that mysql_conn->lock is NOT set on function entry */
 static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 				     storage_field_t *fields, char *ending)
@@ -514,8 +587,11 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 	correct_query[strlen(correct_query)-1] = ';';
 	//info("%d query\n%s", __LINE__, query);
 
-	/* see if we have already done this definition */
-	if (!adding) {
+	/* see if table needs to be altered after db server upgrade */
+	if (!adding && _alter_table_after_upgrade(mysql_conn, query)) {
+		run_update = 3;
+	} else if (!adding && !run_update) {
+		/* see if we have already done this definition */
 		char *quoted = slurm_add_slash_to_quotes(query);
 		char *query2 = xstrdup_printf("select table_name from "
 					      "%s where definition='%s'",
@@ -554,8 +630,13 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 
 		if (run_update == 2)
 			debug4("Table %s doesn't exist, adding", table_name);
+		else if (run_update == 3)
+			info("MariaDB >= 10.2.1 in use with a table from an earlier version or from MySQL. Updating table %s...",
+			      table_name);
 		else
 			debug("Table %s has changed.  Updating...", table_name);
+
+		debug2("query\n%s", query);
 		if (mysql_db_query(mysql_conn, query)) {
 			xfree(query);
 			return SLURM_ERROR;
@@ -570,6 +651,7 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 					table_name, quoted,
 					quoted, now);
 		xfree(quoted);
+		debug3("query\n%s", query2);
 		if (mysql_db_query(mysql_conn, query2)) {
 			xfree(query2);
 			return SLURM_ERROR;
@@ -585,6 +667,48 @@ static int _mysql_make_table_current(mysql_conn_t *mysql_conn, char *table_name,
 	return SLURM_SUCCESS;
 }
 
+void _set_mysql_ssl_opts(MYSQL *db_conn, const char *options)
+{
+	char *tmp_opts, *token, *save_ptr = NULL;
+	const char *key = NULL, *cert = NULL, *ca = NULL, *ca_path = NULL;
+	const char *cipher = NULL;
+
+	if (!options)
+		return;
+
+	tmp_opts = xstrdup(options);
+	token = strtok_r(tmp_opts, ",", &save_ptr);
+	while (token) {
+		char *opt_str, *val_str = NULL;
+
+		opt_str = strtok_r(token, "=", &val_str);
+
+		if (!opt_str || !val_str) {
+			error("Invalid storage option/val");
+			goto next;
+		} else if (!xstrcasecmp(opt_str, "SSL_CERT"))
+			cert = val_str;
+		else if (!xstrcasecmp(opt_str, "SSL_CA"))
+			ca = val_str;
+		else if (!xstrcasecmp(opt_str, "SSL_CAPATH"))
+			ca_path = val_str;
+		else if (!xstrcasecmp(opt_str, "SSL_KEY"))
+			key = val_str;
+		else if (!xstrcasecmp(opt_str, "SSL_CIPHER"))
+			cipher = val_str;
+		else {
+			error("Invalid storage option '%s'", opt_str);
+			goto next;
+		}
+next:
+		token = strtok_r(NULL, ",", &save_ptr);
+	}
+
+	mysql_ssl_set(db_conn, key, cert, ca, ca_path, cipher);
+
+	xfree(tmp_opts);
+}
+
 /* NOTE: Ensure that mysql_conn->lock is set on function entry */
 static int _create_db(char *db_name, mysql_db_info_t *db_info)
 {
@@ -599,6 +723,8 @@ static int _create_db(char *db_name, mysql_db_info_t *db_info)
 		if (!(mysql_db = mysql_init(mysql_db)))
 			fatal("mysql_init failed: %s", mysql_error(mysql_db));
 
+		_set_mysql_ssl_opts(mysql_db, db_info->params);
+
 		db_host = db_info->host;
 		db_ptr = mysql_real_connect(mysql_db,
 					    db_host, db_info->user,
@@ -650,7 +776,8 @@ extern mysql_conn_t *create_mysql_conn(int conn_num, bool rollback,
 {
 	mysql_conn_t *mysql_conn = xmalloc(sizeof(mysql_conn_t));
 
-	mysql_conn->rollback = rollback;
+	if (rollback)
+		mysql_conn->flags |= DB_CONN_FLAG_ROLLBACK;
 	mysql_conn->conn = conn_num;
 	mysql_conn->cluster_name = xstrdup(cluster_name);
 	slurm_mutex_init(&mysql_conn->lock);
@@ -679,25 +806,22 @@ extern mysql_db_info_t *create_mysql_db_info(slurm_mysql_plugin_type_t type)
 
 	switch (type) {
 	case SLURM_MYSQL_PLUGIN_AS:
-		db_info->port = slurm_get_accounting_storage_port();
-		if (!db_info->port) {
-			db_info->port = DEFAULT_MYSQL_PORT;
-			slurm_set_accounting_storage_port(db_info->port);
-		}
-		db_info->host = slurm_get_accounting_storage_host();
-		db_info->backup = slurm_get_accounting_storage_backup_host();
-		db_info->user = slurm_get_accounting_storage_user();
-		db_info->pass = slurm_get_accounting_storage_pass();
+		db_info->port = slurm_conf.accounting_storage_port;
+		db_info->host = xstrdup(slurm_conf.accounting_storage_host);
+		db_info->backup =
+			xstrdup(slurm_conf.accounting_storage_backup_host);
+		db_info->user = xstrdup(slurm_conf.accounting_storage_user);
+		db_info->pass = xstrdup(slurm_conf.accounting_storage_pass);
+		db_info->params = xstrdup(slurm_conf.accounting_storage_params);
 		break;
 	case SLURM_MYSQL_PLUGIN_JC:
-		db_info->port = slurm_get_jobcomp_port();
-		if (!db_info->port) {
-			db_info->port = DEFAULT_MYSQL_PORT;
-			slurm_set_jobcomp_port(db_info->port);
-		}
-		db_info->host = slurm_get_jobcomp_host();
-		db_info->user = slurm_get_jobcomp_user();
-		db_info->pass = slurm_get_jobcomp_pass();
+		if (!slurm_conf.job_comp_port)
+			slurm_conf.job_comp_port = DEFAULT_MYSQL_PORT;
+		db_info->port = slurm_conf.job_comp_port;
+		db_info->host = xstrdup(slurm_conf.job_comp_host);
+		db_info->user = xstrdup(slurm_conf.job_comp_user);
+		db_info->pass = xstrdup(slurm_conf.job_comp_pass);
+		db_info->params = xstrdup(slurm_conf.accounting_storage_params);
 		break;
 	default:
 		xfree(db_info);
@@ -725,9 +849,8 @@ extern int mysql_db_get_db_connection(mysql_conn_t *mysql_conn, char *db_name,
 	bool storage_init = false;
 	char *db_host = db_info->host;
 	unsigned int my_timeout = 30;
-#ifdef MYSQL_OPT_RECONNECT
-	my_bool reconnect = 1;
-#endif
+	bool reconnect = 0;
+
 	xassert(mysql_conn);
 
 	slurm_mutex_lock(&mysql_conn->lock);
@@ -738,18 +861,19 @@ extern int mysql_db_get_db_connection(mysql_conn_t *mysql_conn, char *db_name,
 		      mysql_error(mysql_conn->db_conn));
 	}
 
-	/* If this ever changes you will need to alter
+	mysql_options(mysql_conn->db_conn, MYSQL_OPT_RECONNECT, &reconnect);
+
+	/*
+	 * If this ever changes you will need to alter
 	 * src/common/slurmdbd_defs.c function _send_init_msg to
 	 * handle a different timeout when polling for the
 	 * response.
 	 */
-#ifdef MYSQL_OPT_RECONNECT
-	/* make sure reconnect is on */
-	mysql_options(mysql_conn->db_conn, MYSQL_OPT_RECONNECT,
-		      &reconnect);
-#endif
 	mysql_options(mysql_conn->db_conn, MYSQL_OPT_CONNECT_TIMEOUT,
 		      (char *)&my_timeout);
+
+	_set_mysql_ssl_opts(mysql_conn->db_conn, db_info->params);
+
 	while (!storage_init) {
 		debug2("Attempting to connect to %s:%d", db_host,
 		       db_info->port);
@@ -764,6 +888,14 @@ extern int mysql_db_get_db_connection(mysql_conn_t *mysql_conn, char *db_name,
 				debug("Database %s not created.  Creating",
 				      db_name);
 				rc = _create_db(db_name, db_info);
+
+				/*
+				 * When using ca, cert and key the next
+				 * connect will fail. Setting the options again
+				 * fixes it.
+				 */
+				_set_mysql_ssl_opts(mysql_conn->db_conn,
+						    db_info->params);
 				continue;
 			}
 
@@ -785,7 +917,7 @@ extern int mysql_db_get_db_connection(mysql_conn_t *mysql_conn, char *db_name,
 		}
 
 		storage_init = true;
-		if (mysql_conn->rollback)
+		if (mysql_conn->flags & DB_CONN_FLAG_ROLLBACK)
 			mysql_autocommit(mysql_conn->db_conn, 0);
 		rc = _mysql_query_internal(mysql_conn->db_conn,
 					   "SET session sql_mode='ANSI_QUOTES,"
@@ -976,8 +1108,8 @@ extern uint64_t mysql_db_insert_ret_id(mysql_conn_t *mysql_conn, char *query)
 		new_id = mysql_insert_id(mysql_conn->db_conn);
 		if (!new_id) {
 			/* should have new id */
-			error("We should have gotten a new id: %s",
-			      mysql_error(mysql_conn->db_conn));
+			error("%s: We should have gotten a new id: %s",
+			      __func__, mysql_error(mysql_conn->db_conn));
 		}
 	}
 	slurm_mutex_unlock(&mysql_conn->lock);
@@ -989,7 +1121,7 @@ extern int mysql_db_create_table(mysql_conn_t *mysql_conn, char *table_name,
 				 storage_field_t *fields, char *ending)
 {
 	char *query = NULL;
-	int i = 0, rc;
+	int rc;
 	storage_field_t *first_field = fields;
 
 	if (!fields || !fields->name) {
@@ -1017,13 +1149,11 @@ extern int mysql_db_create_table(mysql_conn_t *mysql_conn, char *table_name,
 
 	query = xstrdup_printf("create table if not exists %s (`%s` %s",
 			       table_name, fields->name, fields->options);
-	i = 1;
 	fields++;
 
 	while (fields && fields->name) {
 		xstrfmtcat(query, ", `%s` %s", fields->name, fields->options);
 		fields++;
-		i++;
 	}
 	xstrcat(query, ending);
 
