@@ -44,7 +44,7 @@
 
 #include "src/common/env.h"
 #include "src/common/slurmdbd_defs.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_time.h"
 #include "src/common/xstring.h"
 #include "src/slurmdbd/read_config.h"
@@ -98,7 +98,7 @@ static void _dump_slurmdb_assoc_records(List assoc_list)
 static void _dump_slurmdb_clus_res_record(slurmdb_clus_res_rec_t *clus_res)
 {
 	debug("\t\t\tname=%s", clus_res->cluster);
-	debug("\t\t\tpercent_allowed=%u", clus_res->percent_allowed);
+	debug("\t\t\tallowed=%u", clus_res->allowed);
 }
 
 static void _dump_slurmdb_clus_res_records(List clus_res_list)
@@ -153,19 +153,14 @@ extern int addto_update_list(List update_list, slurmdb_update_type_t type,
 	slurmdb_res_rec_t *res = object;
 	slurmdb_wckey_rec_t *wckey = object;
 #endif
-	ListIterator itr = NULL;
 
 	if (!update_list) {
 		error("no update list given");
 		return SLURM_ERROR;
 	}
 
-	itr = list_iterator_create(update_list);
-	while((update_object = list_next(itr))) {
-		if (update_object->type == type)
-			break;
-	}
-	list_iterator_destroy(itr);
+	update_object = list_find_first(
+		update_list, slurmdb_find_update_object_in_list, &type);
 
 	if (update_object) {
 		/* here we prepend primarly for remove association
@@ -614,7 +609,7 @@ extern bool is_user_any_coord(void *db_conn, slurmdb_user_rec_t *user)
 }
 
 /*
- * acct_get_db_name - get database name of accouting storage
+ * acct_get_db_name - get database name of accounting storage
  * RET: database name, should be free-ed by caller
  */
 extern char *acct_get_db_name(void)
@@ -898,10 +893,6 @@ extern int archive_write_file(buf_t *buffer, char *cluster_name,
 	new_file = _make_archive_name(period_start, period_end,
 				      cluster_name, arch_dir,
 				      arch_type, archive_period);
-	if (!new_file) {
-		error("%s: Unable to make archive file name.", __func__);
-		return SLURM_ERROR;
-	}
 
 	debug("Storing %s archive for %s at %s",
 	      arch_type, cluster_name, new_file);
