@@ -57,12 +57,12 @@
 
 #include "src/common/slurm_xlator.h"
 #include "src/common/fd.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/acct_gather_profile.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_time.h"
 #include "src/common/xstring.h"
-#include "src/slurmd/common/proctrack.h"
+#include "src/interfaces/proctrack.h"
 #include "hdf5_api.h"
 
 #define HDF5_CHUNK_SIZE 10
