@@ -43,8 +43,16 @@
 
 #include "slurm/slurm_errno.h"
 #include "src/common/slurm_xlator.h"
+
+#include "src/common/data.h"
 #include "src/common/pack.h"
+#include "src/common/slurm_protocol_api.h"
+#include "src/common/read_config.h"
+#include "src/common/run_in_daemon.h"
 #include "src/common/uid.h"
+#include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
+#include "src/interfaces/serializer.h"
 
 #include "auth_jwt.h"
 
@@ -77,7 +85,7 @@ const char plugin_name[] = "JWT authentication plugin";
 const char plugin_type[] = "auth/jwt";
 const uint32_t plugin_id = AUTH_PLUGIN_JWT;
 const uint32_t plugin_version   = SLURM_VERSION_NUMBER;
-bool hash_enable = false;
+const bool hash_enable = false;
 
 typedef struct {
 	int index; /* MUST ALWAYS BE FIRST. DO NOT PACK. */
@@ -98,6 +106,7 @@ typedef struct {
 data_t *jwks = NULL;
 buf_t *key = NULL;
 char *token = NULL;
+static char *claim_field = NULL;
 __thread char *thread_token = NULL;
 __thread char *thread_username = NULL;
 
@@ -117,20 +126,32 @@ __thread char *thread_username = NULL;
  *		requestor for a given username and duration.
  */
 
-const char *jwt_key_field = "jwt_key=";
-const char *jwks_key_field = "jwks=";
+static const char *jwt_key_field = "jwt_key=";
+static const char *jwks_key_field = "jwks=";
 
 static data_for_each_cmd_t _build_jwks_keys(data_t *d, void *arg)
 {
-	char *alg, *kid, *n, *e, *key;
-
-	/* Ignore non-RS256 keys in the JWKS */
-	alg = data_get_string(data_key_get(d, "alg"));
-	if (xstrcasecmp(alg, "RS256"))
-		return DATA_FOR_EACH_CONT;
+	char *alg, *kid, *n, *e, *key, *x5c, *kty;
 
 	if (!(kid = data_get_string(data_key_get(d, "kid"))))
 		fatal("%s: failed to load kid field", __func__);
+
+	alg = data_get_string(data_key_get(d, "alg"));
+	kty = data_get_string(data_key_get(d, "kty"));
+	x5c = data_get_string(data_key_get(d, "x5c"));
+
+	if (kty && x5c && !xstrcasecmp(kty, "RSA") && x5c[0]) {
+		debug3("x5c for kid %s is %s", kid, x5c);
+
+		data_set_int(data_key_set(d, "slurm-pem-len"), strlen(x5c));
+		data_set_string_own(data_key_set(d, "slurm-pem"), x5c);
+		return DATA_FOR_EACH_CONT;
+	}
+
+	/* Ignore non-RS256 keys in the JWKS */
+	if (xstrcasecmp(alg, "RS256"))
+		return DATA_FOR_EACH_CONT;
+
 	if (!(e = data_get_string(data_key_get(d, "e"))))
 		fatal("%s: failed to load e field", __func__);
 	if (!(n = data_get_string(data_key_get(d, "n"))))
@@ -139,8 +160,8 @@ static data_for_each_cmd_t _build_jwks_keys(data_t *d, void *arg)
 	key = pem_from_mod_exp(n, e);
 	debug3("key for kid %s mod %s exp %s is\n%s", kid, n, e, key);
 
-	data_set_string_own(data_key_set(d, "slurm-pem"), key);
 	data_set_int(data_key_set(d, "slurm-pem-len"), strlen(key));
+	data_set_string_own(data_key_set(d, "slurm-pem"), key);
 
 	return DATA_FOR_EACH_CONT;
 }
@@ -153,9 +174,12 @@ static void _init_jwks(void)
 	if (!(begin = xstrstr(slurm_conf.authalt_params, jwks_key_field)))
 		return;
 
-	if (data_init(MIME_TYPE_JSON_PLUGIN, NULL))
+	if (data_init())
 		fatal("%s: data_init() failed", __func__);
 
+	if (serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL))
+		fatal("%s: serializer_g_init() failed", __func__);
+
 	start = begin + strlen(jwks_key_field);
 	if ((end = xstrstr(start, ",")))
 		key_file = xstrndup(start, (end - start));
@@ -168,10 +192,11 @@ static void _init_jwks(void)
 		      plugin_type, key_file);
 	}
 
-	if (data_g_deserialize(&jwks, buf->head, buf->size, MIME_TYPE_JSON))
+	if (serialize_g_string_to_data(&jwks, buf->head, buf->size,
+				       MIME_TYPE_JSON))
 		fatal("%s: failed to deserialize jwks file `%s`",
 		      __func__, key_file);
-	free_buf(buf);
+	FREE_NULL_BUFFER(buf);
 
 	/* force everything to be a string */
 	(void) data_convert_tree(jwks, DATA_TYPE_STRING);
@@ -223,8 +248,24 @@ static void _init_hs256(void)
 extern int init(void)
 {
 	if (running_in_slurmctld() || running_in_slurmdbd()) {
+		char *claim;
+
 		_init_jwks();
 		_init_hs256();
+
+		/*
+		 * Support an optional custom username claim field in addition
+		 * to 'sun' and 'username'.
+		 */
+		if ((claim = xstrstr(slurm_conf.authalt_params, "userclaimfield="))) {
+			char *end;
+
+			claim_field = xstrdup(claim + 15);
+			if ((end = xstrstr(claim_field, ",")))
+				*end = '\0';
+
+			info("Custom user claim field: %s", claim_field);
+		}
 	} else {
 		/* we must be in a client command */
 		token = getenv("SLURM_JWT");
@@ -243,8 +284,9 @@ extern int init(void)
 
 extern int fini(void)
 {
+	xfree(claim_field);
 	FREE_NULL_DATA(jwks);
-	free_buf(key);
+	FREE_NULL_BUFFER(key);
 
 	return SLURM_SUCCESS;
 }
@@ -254,17 +296,14 @@ auth_token_t *auth_p_create(char *auth_info, uid_t r_uid, void *data, int dlen)
 	return xmalloc(sizeof(auth_token_t));
 }
 
-int auth_p_destroy(auth_token_t *cred)
+extern void auth_p_destroy(auth_token_t *cred)
 {
-	if (cred == NULL) {
-		slurm_seterrno(ESLURM_AUTH_MEMORY);
-		return SLURM_ERROR;
-	}
+	if (!cred)
+		return;
 
 	xfree(cred->token);
 	xfree(cred->username);
 	xfree(cred);
-	return SLURM_SUCCESS;
 }
 
 typedef struct {
@@ -408,11 +447,17 @@ int auth_p_verify(auth_token_t *cred, char *auth_info)
 	 * 'username' is used otherwise
 	 */
 	if (!(username = xstrdup(jwt_get_grant(jwt, "sun"))) &&
-	    !(username = xstrdup(jwt_get_grant(jwt, "username")))) {
+	    !(username = xstrdup(jwt_get_grant(jwt, "username"))) &&
+	    (!claim_field ||
+	     !(username = xstrdup(jwt_get_grant(jwt, claim_field)))))
+	{
 		error("%s: jwt_get_grant failure", __func__);
 		goto fail;
 	}
 
+	jwt_free(jwt);
+	jwt = NULL;
+
 	if (!cred->username)
 		cred->username = username;
 	else if (!xstrcmp(cred->username, username)) {
@@ -511,7 +556,7 @@ char *auth_p_get_host(auth_token_t *cred)
 	return NULL;
 }
 
-int auth_p_get_data(auth_token_t *cred, char **data, uint32_t *len)
+extern int auth_p_get_data(auth_token_t *cred, char **data, uint32_t *len)
 {
 	if (cred == NULL) {
 		slurm_seterrno(ESLURM_AUTH_BADARG);
@@ -597,12 +642,19 @@ char *auth_p_token_generate(const char *username, int lifespan)
 	time_t now = time(NULL);
 	jwt_t *jwt;
 	char *token, *xtoken;
+	long grant_time = now + lifespan;
 
 	if (!key) {
 		error("%s: cannot issue tokens, no key loaded", __func__);
 		return NULL;
 	}
 
+	if ((lifespan >= NO_VAL) || (lifespan <= 0) || (grant_time <= 0)) {
+		error("%s: cannot issue token: requested lifespan %ds not supported",
+		      __func__, lifespan);
+		return NULL;
+	}
+
 	if (jwt_new(&jwt)) {
 		error("%s: jwt_new failure", __func__);
 		goto fail;
@@ -612,7 +664,7 @@ char *auth_p_token_generate(const char *username, int lifespan)
 		error("%s: jwt_add_grant_int failure", __func__);
 		goto fail;
 	}
-	if (jwt_add_grant_int(jwt, "exp", now + lifespan)) {
+	if (jwt_add_grant_int(jwt, "exp", grant_time)) {
 		error("%s: jwt_add_grant_int failure", __func__);
 		goto fail;
 	}
@@ -632,6 +684,11 @@ char *auth_p_token_generate(const char *username, int lifespan)
 		goto fail;
 	}
 	xtoken = xstrdup(token);
+	/*
+	 * Ideally this would be jwt_free_str() instead of free(),
+	 * but that function doesn't exist in older versions of libjwt.
+	 */
+	free(token);
 
 	jwt_free(jwt);
 
