@@ -40,7 +40,7 @@
 \*****************************************************************************/
 
 #include "src/common/slurm_xlator.h"
-#include "src/common/slurm_jobacct_gather.h"
+#include "src/interfaces/jobacct_gather.h"
 
 /*
  * These variables are required by the generic plugin interface.  If they
