@@ -41,7 +41,8 @@
 
 #include "slurm/slurm_errno.h"
 
-#include "src/common/slurm_jobcomp.h"
+#include "src/common/slurm_xlator.h"
+#include "src/interfaces/jobcomp.h"
 #include "src/slurmctld/slurmctld.h"
 
 /*
@@ -92,7 +93,7 @@ extern int fini(void)
  * logging API.
  */
 
-extern int jobcomp_p_set_location(char *location)
+extern int jobcomp_p_set_location(void)
 {
 	return SLURM_SUCCESS;
 }
