@@ -52,21 +52,22 @@
 #include "src/common/parse_time.h"
 #include "src/common/proc_args.h"
 #include "src/common/ref.h"
-#include "src/common/select.h"
-#include "src/common/slurm_acct_gather_profile.h"
+#include "src/interfaces/select.h"
+#include "src/interfaces/acct_gather_profile.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_resource_info.h"
 #include "src/common/strlcpy.h"
 #include "src/common/tres_bind.h"
 #include "src/common/tres_frequency.h"
+#include "src/common/uid.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
 #include "src/slurmrestd/operations.h"
 
-#include "src/plugins/openapi/v0.0.36/api.h"
+#include "src/plugins/openapi/v0.0.37/api.h"
 
 typedef struct {
 	const char *param;
@@ -150,7 +151,6 @@ const params_t job_params[] = {
 	{ "kill_command", 'K', true },
 	{ "kill_on_bad_exit", 'K', true },
 	{ "kill_on_invalid_dependency", LONG_OPT_KILL_INV_DEP },
-	{ "kill_on_invalid dependency", LONG_OPT_KILL_INV_DEP },
 	{ "label", 'l', true },
 	{ "license", 'L' },
 	{ "licenses", 'L' },
@@ -188,7 +188,7 @@ const params_t job_params[] = {
 	{ "partition", 'p' },
 	{ "power_flags", LONG_OPT_POWER, true },
 	{ "preserve_environment", 'E', true },
-	{ "priority", LONG_OPT_PRIORITY, true },
+	{ "priority", LONG_OPT_PRIORITY, false },
 	{ "profile", LONG_OPT_PROFILE },
 	{ "prolog", LONG_OPT_PROLOG, true },
 	{ "propagate", LONG_OPT_PROPAGATE, true },
@@ -209,9 +209,7 @@ const params_t job_params[] = {
 	{ "sockets_per_node", LONG_OPT_SOCKETSPERNODE },
 	{ "spread_job", LONG_OPT_SPREAD_JOB },
 	{ "standard_error", 'e' },
-	{ "standard_in", 'i' },
 	{ "standard_input", 'i' },
-	{ "standard_out", 'o' },
 	{ "standard_output", 'o' },
 	{ "task_epilog", LONG_OPT_TASK_EPILOG, true },
 	{ "task_prolog", LONG_OPT_TASK_PROLOG, true },
@@ -271,22 +269,6 @@ static void _list_delete_job_desc_msg_t(void *_p)
 	slurm_free_job_desc_msg(_p);
 }
 
-#define _parse_error(...)                                                    \
-	do {                                                                 \
-		const char *error_string = xstrdup_printf(__VA_ARGS__);      \
-		error("%s", error_string);                                   \
-		data_t *error = data_list_append(errors);                    \
-		data_set_dict(error);                                        \
-		data_set_string(data_key_set(error, "error"), error_string); \
-		xfree(error_string);                                         \
-		if (errno) {                                                 \
-			rc = errno;                                          \
-			errno = 0;                                           \
-		} else                                                       \
-			rc = SLURM_ERROR;                                    \
-		data_set_int(data_key_set(error, "error_code"), rc);         \
-	} while (0)
-
 typedef struct {
 	slurm_opt_t *opt;
 	data_t *errors;
@@ -308,20 +290,22 @@ static data_for_each_cmd_t _per_job_param(const char *key, const data_t *data,
 	xstrtolower(lkey);
 
 	if (!(rc = hsearch_r(e, FIND, &re, &hash_params))) {
-		_parse_error("Unknown key \"%s\": %m", lkey);
+		resp_error(errors, rc, "hsearch_r",
+			   "Unknown key \"%s\"", lkey);
 		return DATA_FOR_EACH_FAIL;
 	}
 
 	p = re->data;
 	if (p->disabled) {
-		_parse_error("Disabled key: \"%s\"", p->param);
+		resp_error(errors, rc, "openapi specification",
+			   "Disabled key: \"%s\"", p->param);
 		return DATA_FOR_EACH_FAIL;
 	}
 
 	if ((rc = slurm_process_option_data(args->opt, p->optval, data,
 					    errors))) {
-		_parse_error("process failed for key %s with error: %s",
-			     key, slurm_strerror(rc));
+		resp_error(errors, rc, "slurm_process_option_data",
+			   "Unknown key \"%s\"", lkey);
 		return DATA_FOR_EACH_FAIL;
 	}
 
@@ -339,11 +323,8 @@ static int _fill_job_desc_from_opts(slurm_opt_t *opt, job_desc_msg_t *desc)
 	if (!desc)
 		return -1;
 
-	if (!opt->job_name)
-		desc->name = xstrdup("openapi");
-
 	desc->array_inx = xstrdup(sbopt->array_inx);
-	desc->batch_features = sbopt->batch_features;
+	desc->batch_features = xstrdup(sbopt->batch_features);
 	desc->container = xstrdup(opt->container);
 
 	desc->wait_all_nodes = sbopt->wait_all_nodes;
@@ -380,11 +361,13 @@ static int _fill_job_desc_from_opts(slurm_opt_t *opt, job_desc_msg_t *desc)
 	desc->env_size = envcount(desc->environment);
 
 	/* Disable sending uid/gid as it is handled by auth layer */
-	desc->user_id = NO_VAL;
-	desc->group_id = NO_VAL;
+	desc->user_id = SLURM_AUTH_NOBODY;
+	desc->group_id = SLURM_AUTH_NOBODY;
 
-	desc->argc     = sbopt->script_argc;
-	desc->argv     = sbopt->script_argv;
+	desc->argc = opt->argc;
+	desc->argv = xcalloc(opt->argc, sizeof(*desc->argv));
+	for (int i = 0; i < opt->argc; i++)
+		desc->argv[i] = xstrdup(opt->argv[i]);
 	desc->std_err  = xstrdup(opt->efname);
 	desc->std_in   = xstrdup(opt->ifname);
 	desc->std_out  = xstrdup(opt->ofname);
@@ -420,14 +403,14 @@ static job_desc_msg_t *_parse_job_desc(const data_t *job, data_t *errors,
 	}
 
 	req = slurm_opt_create_job_desc(&opt, !update_only);
-	if (!update_only)
-		req->task_dist = SLURM_DIST_UNKNOWN;
-
-	if (_fill_job_desc_from_opts(&opt, req)) {
+	if (_fill_job_desc_from_opts(&opt, req) == -1) {
 		rc = SLURM_ERROR;
 		goto cleanup;
 	}
 
+	if (!update_only)
+		req->task_dist = SLURM_DIST_UNKNOWN;
+
 	if (!update_only && (!req->environment || !req->env_size)) {
 		/*
 		 * Jobs provided via data must have their environment
@@ -534,15 +517,10 @@ static data_for_each_cmd_t _parse_job_component(const data_t *data, void *arg)
 
 		list_append(rc->jobs, job_desc);
 	} else { /* parsing failed */
-		data_t *error = data_list_append(j->errors);
-		char *error_string = xstrdup_printf(
-			"%s: unexpected failure parsing het job: %zd",
-			__func__, j->i);
-		data_set_dict(error);
-		data_set_string(data_key_set(error, "error"), error_string);
-		//error("%s", error_string);
-		xfree(error_string);
-		rc->rc = ESLURM_REST_FAIL_PARSING;
+		rc->rc = resp_error(j->errors, ESLURM_REST_FAIL_PARSING,
+				    "_parse_job_desc",
+				    "unexpected failure parsing het job: %zd",
+				    j->i);
 		return DATA_FOR_EACH_FAIL;
 	}
 
@@ -998,7 +976,15 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 	data_set_string(data_key_set(jd, "tres_alloc_str"),
 			job->tres_alloc_str);
 	data_set_int(data_key_set(jd, "user_id"), job->user_id);
-	data_set_string(data_key_set(jd, "user_name"), job->user_name);
+
+	if (job->user_name) {
+		data_set_string(data_key_set(jd, "user_name"), job->user_name);
+	} else {
+		data_set_string_own(
+			data_key_set(jd, "user_name"),
+			uid_to_string_or_null((uid_t) job->user_id));
+	}
+
 	/* wait4switch intentionally omitted */
 	data_set_string(data_key_set(jd, "wckey"), job->wckey);
 	data_set_string(data_key_set(jd, "current_working_directory"),
@@ -1013,50 +999,53 @@ static int _op_handler_jobs(const char *context_id,
 {
 	int rc = SLURM_SUCCESS;
 	job_info_msg_t *job_info_ptr = NULL;
-	(void) populate_response_format(resp);
+	data_t *errors = populate_response_format(resp);
 	data_t *jobs = data_set_list(data_key_set(resp, "jobs"));
+	time_t update_time = 0; /* default to unix epoch */
 
 	debug4("%s: jobs handler called by %s", __func__, context_id);
 
-	rc = slurm_load_jobs((time_t)NULL, &job_info_ptr,
-			     SHOW_ALL|SHOW_DETAIL);
+	if ((rc = get_date_param(query, "update_time", &update_time)))
+	    goto done;
 
-	if (rc == SLURM_SUCCESS && job_info_ptr &&
-	    job_info_ptr->record_count) {
+	rc = slurm_load_jobs(update_time, &job_info_ptr,
+			     SHOW_ALL | SHOW_DETAIL);
+
+	if (rc == SLURM_NO_CHANGE_IN_DATA) {
+		/* no-op: nothing to do here */
+	} else if ((rc == SLURM_SUCCESS) && job_info_ptr &&
+		   job_info_ptr->record_count) {
 		for (size_t i = 0; i < job_info_ptr->record_count; ++i) {
 			dump_job_info(job_info_ptr->job_array + i,
 				      data_list_append(jobs));
 		}
+	} else if (rc) {
+		resp_error(errors, rc, "slurm_load_jobs",
+			   "Failed while looking for jobs");
 	}
 
+done:
 	slurm_free_job_info_msg(job_info_ptr);
 
 	return rc;
 }
 
-#define _job_error(...)                                                      \
-	do {                                                                 \
-		const char *error_string = xstrdup_printf(__VA_ARGS__);      \
-		error("%s", error_string);                                   \
-		data_t *error = data_list_append(errors);                    \
-		data_set_dict(error);                                        \
-		data_set_string(data_key_set(error, "error"), error_string); \
-		xfree(error_string);                                         \
-		if (errno) {                                                 \
-			rc = errno;                                          \
-			errno = 0;                                           \
-		} else                                                       \
-			rc = SLURM_ERROR;                                    \
-		data_set_int(data_key_set(error, "error_code"), errno);      \
-	} while (0)
-
 static int _handle_job_get(const char *context_id, http_request_method_t method,
 			   data_t *parameters, data_t *query, int tag,
-			   data_t *resp, const uint32_t job_id,
+			   data_t *resp, const char *job_id_str,
 			   data_t *const errors)
 {
 	int rc = SLURM_SUCCESS;
 	job_info_msg_t *job_info_ptr = NULL;
+	uint32_t job_id = slurm_xlate_job_id((char *) job_id_str);
+
+	if (!job_id) {
+		rc = ESLURM_REST_INVALID_JOBS_DESC;
+		resp_error(errors, rc, "_handle_job_get",
+			   "Unable to find JobId=%s", job_id_str);
+		return rc;
+	}
+
 	rc = slurm_load_job(&job_info_ptr, job_id, SHOW_ALL|SHOW_DETAIL);
 	data_t *jobs = data_set_list(data_key_set(resp, "jobs"));
 
@@ -1066,7 +1055,8 @@ static int _handle_job_get(const char *context_id, http_request_method_t method,
 				      data_list_append(jobs));
 		}
 	} else {
-		_job_error("%s: unknown job %d", __func__, job_id);
+		resp_error(errors, rc, "slurm_load_job",
+			   "Unable to find JobId=%s", job_id_str);
 	}
 
 	slurm_free_job_info_msg(job_info_ptr);
@@ -1077,19 +1067,17 @@ static int _handle_job_get(const char *context_id, http_request_method_t method,
 static int _handle_job_delete(const char *context_id,
 			      http_request_method_t method,
 			      data_t *parameters, data_t *query, int tag,
-			      data_t *resp, const uint32_t job_id,
+			      data_t *resp, const char *job_id,
 			      data_t *const errors, const int signal)
 {
-	if (slurm_kill_job(job_id, signal, KILL_FULL_JOB)) {
-		int rc;
-
+	if (slurm_kill_job2(job_id, signal, KILL_FULL_JOB, NULL)) {
 		/* Already signaled jobs are considered a success here */
 		if (errno == ESLURM_ALREADY_DONE)
 			return SLURM_SUCCESS;
 
-		_job_error("%s: unable to kill job %d with signal %d: %s",
-			   __func__, job_id, signal, slurm_strerror(errno));
-		return rc;
+		return resp_error(errors, errno,"slurm_kill_job",
+				  "unable to kill JobId=%s with signal %d: %s",
+				  job_id, signal, slurm_strerror(errno));
 	}
 
 	return SLURM_SUCCESS;
@@ -1098,52 +1086,54 @@ static int _handle_job_delete(const char *context_id,
 static int _handle_job_post(const char *context_id,
 			    http_request_method_t method, data_t *parameters,
 			    data_t *query, int tag, data_t *resp,
-			    const uint32_t job_id, data_t *const errors)
+			    const char *job_id, data_t *const errors)
 {
 	int rc = SLURM_SUCCESS;
+	job_parse_list_t jobs_rc;
 
 	if (get_log_level() >= LOG_LEVEL_DEBUG5) {
 		char *buffer = NULL;
 
-		data_g_serialize(&buffer, query, MIME_TYPE_JSON,
-				 DATA_SER_FLAGS_COMPACT);
-		debug5("%s: job update from %s: %s", __func__, context_id,
-		       buffer);
-
+		serialize_g_data_to_string(&buffer, NULL, query, MIME_TYPE_JSON,
+					   SER_FLAGS_COMPACT);
+		debug5("%s: job update from %s: %s",
+		       __func__, context_id, buffer);
 		xfree(buffer);
 	}
 
-	job_parse_list_t jobs_rc = _parse_job_list(query, NULL, errors,
-						   true);
+	jobs_rc = _parse_job_list(query, NULL, errors, true);
 
 	if (jobs_rc.rc) {
-		_job_error("%s: job parsing failed for %s",
-			   __func__, context_id);
+		resp_error(errors, jobs_rc.rc, "_parse_job_list",
+			   "job parsing failed for %s", context_id);
 	} else {
 		debug3("%s: job parsing successful for %s",
 		       __func__, context_id);
 		rc = jobs_rc.rc;
 		if (jobs_rc.het_job) {
-			_job_error("%s: unexpected het job request from %s",
-				   __func__, context_id);
+			resp_error(errors, rc, "_parse_job_list",
+				   "unexpected het job request from %s",
+				   context_id);
 		} else {
 			job_array_resp_msg_t *resp = NULL;
 			errno = 0;
-			jobs_rc.job->job_id = job_id;
-			debug5("%s: sending job_id:%d update for %s",
+			jobs_rc.job->job_id_str = xstrdup(job_id);
+			debug5("%s: sending JobId=%s update for %s",
 			       __func__, job_id, context_id);
 
 			rc = slurm_update_job2(jobs_rc.job, &resp);
 
-			if (rc)
-				_job_error("%s: job update from %s failed: %s",
-					   __func__, context_id,
-					   slurm_strerror(errno));
-			else if (resp && resp->error_code && resp->error_code)
-				_job_error(
-					"%s: job array update from %s failed with error_code: %s",
-					__func__, context_id,
-					slurm_strerror(*resp->error_code));
+			if (rc) {
+				resp_error(errors, errno, "_parse_job_list",
+					   "job update from %s failed",
+					   context_id);
+			} else if (resp && resp->error_code &&
+				   resp->error_code) {
+				resp_error(errors, *resp->error_code,
+					   "_parse_job_list",
+					   "job array update from %s failed",
+					   context_id);
+			}
 
 			slurm_free_job_desc_msg(jobs_rc.job);
 			slurm_free_job_array_resp(resp);
@@ -1159,33 +1149,42 @@ static int _op_handler_job(const char *context_id, http_request_method_t method,
 {
 	int rc = SLURM_SUCCESS;
 	data_t *data_jobid;
-	uint32_t job_id = 0;
+
+	const char *job_id_str = NULL;
 	data_t *errors = populate_response_format(resp);
 	debug4("%s: job handler %s called by %s with tag %d",
 	       __func__, get_http_method_string(method), context_id, tag);
 
-	if (!parameters)
-		_job_error("%s: [%s] missing request parameters",
-			   __func__, context_id);
-	else if (!(data_jobid = data_key_get(parameters, "job_id")))
-		_job_error("%s: [%s] missing job_id in parameters",
-			   __func__, context_id);
-	else if (data_get_type(data_jobid) != DATA_TYPE_INT_64)
-		_job_error("%s: [%s] invalid job_id data type",
-			   __func__, context_id);
-	else if (data_get_int(data_jobid) == 0)
-		_job_error("%s: [%s] job_id is zero",
-			   __func__, context_id);
-	else if (data_get_int(data_jobid) >= NO_VAL)
-		_job_error("%s: [%s] job_id too large: %" PRId64,
-			   __func__, context_id, data_get_int(data_jobid));
-	else
-		job_id = data_get_int(data_jobid);
+	if (!parameters) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] missing request parameters",
+				  context_id);
+	} else if (!(data_jobid = data_key_get(parameters, "job_id"))) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] missing job_id in parameters",
+				  context_id);
+	} else if (data_convert_type(data_jobid, DATA_TYPE_STRING) !=
+		   DATA_TYPE_STRING) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] job_id is invalid",
+				  context_id);
+	} else {
+		job_id_str = data_get_string(data_jobid);
+	}
 
 	if (rc) {
+		/* do nothing */
+	} else if (!job_id_str || !job_id_str[0]) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] job_id is empty",
+				  context_id);
 	} else if (tag == URL_TAG_JOB && method == HTTP_REQUEST_GET) {
 		rc = _handle_job_get(context_id, method, parameters, query, tag,
-				     resp, job_id, errors);
+				     resp, job_id_str, errors);
 	} else if (tag == URL_TAG_JOB &&
 		   method == HTTP_REQUEST_DELETE) {
 		int signal = 0;
@@ -1198,18 +1197,23 @@ static int _op_handler_job(const char *context_id, http_request_method_t method,
 		else
 			signal = SIGKILL;
 
-		if (signal < 1 || signal >= SIGRTMAX)
-			_job_error("%s: invalid signal: %d", __func__, signal);
-		else
+		if (signal < 1 || signal >= SIGRTMAX) {
+			rc = resp_error(errors, ESLURM_REST_INVALID_QUERY,
+					"HTTP request: signal",
+					"invalid signal: %d", signal);
+		} else {
 			rc = _handle_job_delete(context_id, method, parameters,
-						query, tag, resp, job_id,
+						query, tag, resp, job_id_str,
 						errors, signal);
+		}
 	} else if (tag == URL_TAG_JOB &&
 		   method == HTTP_REQUEST_POST) {
 		rc = _handle_job_post(context_id, method, parameters, query,
-				      tag, resp, job_id, errors);
+				      tag, resp, job_id_str, errors);
 	} else {
-		_job_error("%s: unknown request", __func__);
+		rc = resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				"HTTP request", "%s: unknown request",
+				context_id);
 	}
 
 	return rc;
@@ -1234,14 +1238,14 @@ static int _op_handler_submit_job_post(const char *context_id,
 	if (get_log_level() >= LOG_LEVEL_DEBUG5) {
 		char *buffer = NULL;
 
-		data_g_serialize(&buffer, query, MIME_TYPE_JSON,
-				 DATA_SER_FLAGS_COMPACT);
+		serialize_g_data_to_string(&buffer, NULL, query, MIME_TYPE_JSON,
+					   SER_FLAGS_COMPACT);
 		debug5("%s: job submit query from %s: %s",
 		       __func__, context_id, buffer);
 		xfree(buffer);
 
-		data_g_serialize(&buffer, parameters, MIME_TYPE_JSON,
-				 DATA_SER_FLAGS_COMPACT);
+		serialize_g_data_to_string(&buffer, NULL, parameters,
+					   MIME_TYPE_JSON, SER_FLAGS_COMPACT);
 		debug5("%s: job submit parameters from %s: %s",
 		       __func__, context_id, buffer);
 		xfree(buffer);
@@ -1280,7 +1284,7 @@ static int _op_handler_submit_job_post(const char *context_id,
 				if (slurm_submit_batch_het_job(jobs_rc.jobs,
 							       &resp))
 					rc = errno;
-				list_destroy(jobs_rc.jobs);
+				FREE_NULL_LIST(jobs_rc.jobs);
 			} else {
 				if (slurm_submit_batch_job(jobs_rc.job, &resp))
 					rc = errno;
@@ -1348,8 +1352,8 @@ finish:
 
 static int _op_handler_submit_job(const char *context_id,
 				  http_request_method_t method,
-				  data_t *parameters, data_t *query, int tag,
-				  data_t *resp, void *auth)
+				  data_t *parameters, data_t *query,
+				  int tag, data_t *resp, void *auth)
 {
 	int rc = SLURM_SUCCESS;
 	data_t *errors = populate_response_format(resp);
@@ -1405,12 +1409,12 @@ extern void init_op_jobs(void)
 			      __func__);
 	}
 
-	bind_operation_handler("/slurm/v0.0.36/jobs/", _op_handler_jobs,
-			       URL_TAG_JOBS);
-	bind_operation_handler("/slurm/v0.0.36/job/{job_id}", _op_handler_job,
-			       URL_TAG_JOB);
-	bind_operation_handler("/slurm/v0.0.36/job/submit",
+	bind_operation_handler("/slurm/v0.0.37/job/submit",
 			       _op_handler_submit_job, URL_TAG_JOB_SUBMIT);
+	bind_operation_handler("/slurm/v0.0.37/jobs/", _op_handler_jobs,
+			       URL_TAG_JOBS);
+	bind_operation_handler("/slurm/v0.0.37/job/{job_id}", _op_handler_job,
+			       URL_TAG_JOB);
 }
 
 extern void destroy_op_jobs(void)
