@@ -180,33 +180,38 @@ static char *_get_home(uid_t uid)
 {
 	struct passwd pwd, *pwd_ptr = NULL;
 	char pwd_buf[PW_BUF_SIZE];
+	int rc;
 
-	if (slurm_getpwuid_r(uid, &pwd, pwd_buf, PW_BUF_SIZE, &pwd_ptr)
-	    || (pwd_ptr == NULL)) {
-		error("%s: getpwuid_r(%u):%m", __func__, uid);
+	rc = slurm_getpwuid_r(uid, &pwd, pwd_buf, PW_BUF_SIZE, &pwd_ptr);
+	if (rc || !pwd_ptr) {
+		if (!pwd_ptr && !rc)
+			error("%s: getpwuid_r(%u): no record found",
+			      __func__, uid);
+		else
+			error("%s: getpwuid_r(%u): %m", __func__, uid);
 		return NULL;
 	}
 
 	return xstrdup(pwd.pw_dir);
 }
 
-extern int shutdown_x11_forward(stepd_step_rec_t *job)
+extern int shutdown_x11_forward(stepd_step_rec_t *step)
 {
 	int rc = SLURM_SUCCESS;
 
 	debug("x11 forwarding shutdown in progress");
 	eio_signal_shutdown(eio_handle);
 
-	if (job->x11_xauthority) {
+	if (step->x11_xauthority) {
 		if (local_xauthority) {
-			if (unlink(job->x11_xauthority)) {
+			if (unlink(step->x11_xauthority)) {
 				error("%s: problem unlinking xauthority file %s: %m",
-				      __func__, job->x11_xauthority);
+				      __func__, step->x11_xauthority);
 				rc = SLURM_ERROR;
 			}
 		} else
-			rc = x11_delete_xauth(job->x11_xauthority, hostname,
-					      job->x11_display);
+			rc = x11_delete_xauth(step->x11_xauthority, hostname,
+					      step->x11_display);
 	}
 
 	info("x11 forwarding shutdown complete");
@@ -220,7 +225,7 @@ extern int shutdown_x11_forward(stepd_step_rec_t *job)
  * IN: job
  * OUT: SLURM_SUCCESS or SLURM_ERROR
  */
-extern int setup_x11_forward(stepd_step_rec_t *job)
+extern int setup_x11_forward(stepd_step_rec_t *step)
 {
 	int listen_socket = -1;
 	uint16_t port;
@@ -240,38 +245,38 @@ extern int setup_x11_forward(stepd_step_rec_t *job)
 		.readable = _x11_socket_readable,
 		.handle_read = _x11_socket_read,
 	};
-	srun_info_t *srun = list_peek(job->sruns);
+	srun_info_t *srun = list_peek(step->sruns);
 	/* This should always be set to something else we have a bug. */
 	xassert(srun && srun->protocol_version);
 	protocol_version = srun->protocol_version;
 
-	job_id = job->step_id.job_id;
-	job_uid = job->uid;
-	x11_target = xstrdup(job->x11_target);
-	x11_target_port = job->x11_target_port;
+	job_id = step->step_id.job_id;
+	job_uid = step->uid;
+	x11_target = xstrdup(step->x11_target);
+	x11_target_port = step->x11_target_port;
 
-	slurm_set_addr(&alloc_node, job->x11_alloc_port, job->x11_alloc_host);
+	slurm_set_addr(&alloc_node, step->x11_alloc_port, step->x11_alloc_host);
 
 	debug("X11Parameters: %s", slurm_conf.x11_params);
 
 	if (xstrcasestr(slurm_conf.x11_params, "home_xauthority")) {
 		char *home = NULL;
-		if (!(home = _get_home(job->uid))) {
+		if (!(home = _get_home(step->uid))) {
 			error("could not find HOME in environment");
 			goto shutdown;
 		}
-		job->x11_xauthority = xstrdup_printf("%s/.Xauthority", home);
+		step->x11_xauthority = xstrdup_printf("%s/.Xauthority", home);
 		xfree(home);
 	} else {
 		/* use a node-local XAUTHORITY file instead of ~/.Xauthority */
 		int fd;
 		local_xauthority = true;
-		job->x11_xauthority = slurm_get_tmp_fs(conf->node_name);
-		xstrcat(job->x11_xauthority, "/.Xauthority-XXXXXX");
+		step->x11_xauthority = slurm_get_tmp_fs(conf->node_name);
+		xstrcat(step->x11_xauthority, "/.Xauthority-XXXXXX");
 
 		/* protect against weak file permissions in old glibc */
 		umask(0077);
-		if ((fd = mkstemp(job->x11_xauthority)) == -1) {
+		if ((fd = mkstemp(step->x11_xauthority)) == -1) {
 			error("%s: failed to create temporary XAUTHORITY file: %m",
 			      __func__);
 			goto shutdown;
@@ -292,10 +297,10 @@ extern int setup_x11_forward(stepd_step_rec_t *job)
 		goto shutdown;
 	}
 
-	job->x11_display = port - X11_TCP_PORT_OFFSET;
+	step->x11_display = port - X11_TCP_PORT_OFFSET;
 
 	info("X11 forwarding established on DISPLAY=%s:%d.0",
-	     hostname, job->x11_display);
+	     hostname, step->x11_display);
 
 	eio_handle = eio_handle_create(0);
 	obj = eio_obj_create(listen_socket, &x11_socket_ops, NULL);
@@ -306,8 +311,8 @@ extern int setup_x11_forward(stepd_step_rec_t *job)
 
 shutdown:
 	xfree(x11_target);
-	job->x11_display = 0;
-	xfree(job->x11_xauthority);
+	step->x11_display = 0;
+	xfree(step->x11_xauthority);
 	if (listen_socket != -1)
 		close(listen_socket);
 
