@@ -8,7 +8,7 @@ from pprint import pprint
 
 s = slurm(openapi_client.ApiClient(openapi_client.Configuration()))
 try:
-	pprint(s.slurmdbd_diag())
+	pprint(s.slurmdb_v0039_diag())
 except ApiException as e:
 	print("Exception when calling: %s\n" % e)
 	os._exit(1)
