@@ -3,8 +3,8 @@ import time
 import os
 import openapi_client
 from openapi_client.rest import ApiException
-from openapi_client.models import V0037JobSubmission as jobSubmission
-from openapi_client.models import V0037JobProperties as jobProperties
+from openapi_client.models.v0039_job_submission import V0039JobSubmission as jobSubmission
+from openapi_client.models import V0039JobProperties as jobProperties
 from openapi_client.api import SlurmApi as slurm
 from pprint import pprint
 
@@ -34,7 +34,7 @@ jobProperties(
 ),
 ]
 try:
-	njob = s.slurmctld_submit_job(job)
+	njob = s.slurm_v0039_submit_job(job)
 	pprint(njob)
 except ApiException as e:
 	print("Exception when calling: %s\n" % e)
